--------------------------------------------------------------------------------
-- SubModule GPSTransmitter
-- Created   12/19/2007 6:15:17 PM
--------------------------------------------------------------------------------
library IEEE;
use IEEE.Std_Logic_1164.all;
use IEEE.Numeric_Std.all;

entity GPSTransmitter is port
   (
     CLK       : in    std_logic;
     RST       : in    std_logic;
     INIT      : in    std_logic;
     D         : out   std_logic_vector(7 downto 0);
     LD        : out   std_logic;
     BUSY      : in    std_logic
   );
end GPSTransmitter;
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
architecture Structure of GPSTransmitter is

-- Constant Declarations
   constant ini : string := "@@Aw" & nul & '6' & cr & lf &
                            "@@Hb" & soh & '+' & cr & lf;
   constant ini_len : positive := ini'length(1);

-- Type Declaration
   type FSM_state is (IDLE, SEND, WAITING);

-- Signal Declarations
   signal cnt                : unsigned (11 downto 0);
   signal state              : FSM_state;

-- Functions
   function get_byte (s : in string; i : positive) return std_logic_vector is
      variable c : character;
   begin
      c := s (i);
      return std_logic_vector (To_unsigned (character'pos(c), 8));
   end function get_byte;

begin
   process (CLK, RST) is
   begin
      if RST = '1' then
         state <= IDLE;
         cnt <= To_unsigned (0, 12);
         D <= (others => '0');
         LD <= '0';
      elsif rising_edge (CLK) then
         LD <= '0';
         case state is
            when IDLE =>
               if INIT = '1' then
                  state <= SEND;
                  cnt <= To_unsigned (0, 12);
               end if;
            when SEND =>
               if BUSY = '0' then
                  D <= get_byte (ini, To_integer (cnt)+1);
                  LD <= '1';
                  cnt <= cnt + 1;
                  if cnt = To_unsigned (ini_len - 1, 12) then
                     state <= IDLE;
                  else
                     state <= WAITING;
                  end if;
               end if;
            when WAITING =>
              state <= SEND;
         end case;
      end if;
   end process;

end Structure;
--------------------------------------------------------------------------------
