--------------------------------------------------------------------------------
-- SubModule MasterLink
-- Created   8/1/2008 2:14:39 PM
--------------------------------------------------------------------------------
Library IEEE;
Use IEEE.Std_Logic_1164.all;
Use IEEE.Numeric_Std.all;

entity MasterLink is port
   (
     CLK           : in    std_logic;
     RST           : in    std_logic;

     START         : in    std_logic;
     ENABLE        : in    std_logic;
     RDY           : out   std_logic;
     DONE          : out   std_logic;
     DO            : out   std_logic_vector(7 downto 0);

     MA            : out   std_logic_vector(3 downto 0);
     MD            : in    std_logic_vector(31 downto 0);
     GPSA          : out   std_logic_vector(2 downto 0);
     GPSD          : in    std_logic_vector(31 downto 0);
     EXTA          : out   std_logic_vector(2 downto 0);
     EXTD          : in    std_logic_vector(31 downto 0);
     FOA           : out   std_logic_vector(4 downto 0);
     FOD           : in    std_logic_vector(31 downto 0)
   );
end MasterLink;
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
architecture Structure of MasterLink is

-- Component Declarations

-- Type Declarations
   type state_type  is (IDLE, MASTER_DATA, GPS_DATA, FO_DATA, EXT_DATA);

-- Signal Declarations
   signal state          : state_type;
   signal addr           : unsigned (6 downto 0);
   signal done_flag      : std_logic;
   signal o              : std_logic_vector (31 downto 0);
   signal baddr          : std_logic_vector (1 downto 0);

begin

   -- output state machine
   wr: process (CLK) is
   begin
      if rising_edge (CLK) then
         if RST = '1' then
            state <= IDLE;
            done_flag <= '1';
         elsif START = '1' then
            state <= MASTER_DATA;
            addr <= To_Unsigned (0, 7);
            done_flag <= '0';
         else
            case state is
               when IDLE =>
                  done_flag <= '1';
               when MASTER_DATA =>
                  if ENABLE = '1' then
                     addr <= addr + 1;
                     if addr = To_Unsigned (63, 7) then
                        state <= FO_DATA;
                        addr <= To_Unsigned (0, 7);
                     end if;
                  end if;
               when FO_DATA =>
                  if ENABLE = '1' then
                     addr <= addr + 1;
                     if addr = To_Unsigned (127, 7) then
                        state <= GPS_DATA;
                        addr <= To_Unsigned (0, 7);
                     end if;
                  end if;
               when GPS_DATA =>
                  if ENABLE = '1' then
                     addr <= addr + 1;
                     if addr = To_Unsigned (31, 7) then
                        state <= EXT_DATA;
                        addr <= To_Unsigned (0, 7);
                     end if;
                  end if;
               when EXT_DATA =>
                  if ENABLE = '1' then
                     addr <= addr + 1;
                     if addr = To_Unsigned (31, 7) then
                        state <= IDLE;
                        addr <= To_Unsigned (0, 7);
                        done_flag <= '1';
                     end if;
                  end if;
            end case;
         end if;
      end if;
   end process wr;
   RDY <= not done_flag;
   DONE <= done_flag;

   -- set addresses
   MA <= std_logic_vector (addr (5 downto 2));
   GPSA <= std_logic_vector (addr (4 downto 2));
   EXTA <= std_logic_vector (addr (4 downto 2));
   FOA <= std_logic_vector (addr (6 downto 2));

   -- mux output
   omux: process (state, MD, FOD, GPSD, EXTD) is
   begin
      case state is
         when MASTER_DATA =>
            O <= MD;
         when FO_DATA =>
            O <= FOD;
         when GPS_DATA =>
            O <= GPSD;
         when EXT_DATA =>
            O <= EXTD;
         when others =>
            O <= (others => '0');
      end case;
   end process omux;

   -- mux out bytes (MSB first)
   baddr <= std_logic_vector (addr (1 downto 0));
   obyte: process (baddr, O) is
   begin
      case baddr is
         when B"00" =>
            DO <= O(31 downto 24);
         when B"01" =>
            DO <= O(23 downto 16);
         when B"10" =>
            DO <= O(15 downto 8);
         when others =>
            DO <= O(7 downto 0);
      end case;
   end process obyte;

end Structure;
--------------------------------------------------------------------------------

